/* GUI.java
--------------------------------------------------------------------------------
Main entry point for the GUI.  This must be runnable as we start it on its own
thread.
*/

package gui;

import main.*;
import client.*;
import gui.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

import static main.Main.servers;

public final class GUI implements Runnable, ActionListener, WindowListener
{
	JFrame frame;
	JTable serverList;
	JScrollPane serverListPane;
	JProgressBar queryProgress;
	JLabel statusLabel;
	JMenuBar menuBar;
	JMenu[] menu;
	JMenuItem[] menuItem;

	JButton[] refresh = new JButton[3];

	public GUI()
	{
		try //set look and feel
		{
			if(Config.adaptToSystem)
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			else
				JFrame.setDefaultLookAndFeelDecorated(true);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			Main.crash("Unable to set the look and feel.  Try disabling \"adaptToSystem\".");
		}
		frame = new JFrame("Skulltag Server Browser");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(new Dimension(Config.windowWidth, Config.windowHeight));
		frame.setLocation(new Point(Config.windowX, Config.windowY));
		frame.addWindowListener(this);

		//Server List
		ToolTipManager.sharedInstance().setInitialDelay(0);
		ToolTipManager.sharedInstance().setReshowDelay(0);
		ServerListModel serverListModel = new ServerListModel();
		serverList = new ServerTable(serverListModel);
		serverList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		serverList.setFillsViewportHeight(true);
		TableRowSorter<ServerListModel> sorter = new TableRowSorter<ServerListModel>(serverListModel);
		sorter.setComparator(0, new PlayersImage());
		serverList.setRowSorter(sorter);
		serverList.addMouseListener((ServerListModel) serverList.getModel());
		serverListPane = new JScrollPane(serverList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		Enumeration<TableColumn> columns = serverList.getColumnModel().getColumns();
		while(columns.hasMoreElements())
		{
			TableColumn column = columns.nextElement();
			if(column.getModelIndex() == ServerListModel.COL_PLAYERS)
			{
				column.setCellRenderer(new TableRenderer());
				column.setMinWidth(70);
			}
			if(column.getModelIndex() == ServerListModel.COL_PING)
			{
				column.setMinWidth(35);
				column.setMaxWidth(35);
			}
			else if(column.getModelIndex() == ServerListModel.COL_NAME)
				column.setMinWidth(250);
			else if(column.getModelIndex() == ServerListModel.COL_IP)
			{
				column.setMinWidth(155);
				column.setMaxWidth(155);
			}
			else if(column.getModelIndex() == ServerListModel.COL_MAP)
			{
				column.setMinWidth(55);
				column.setMaxWidth(55);
			}
			else if(column.getModelIndex() == ServerListModel.COL_IWAD)
			{
				column.setMaxWidth(50);
			}
			else if(column.getModelIndex() == ServerListModel.COL_GAMETYPE)
			{
				column.setMinWidth(90);
				column.setMaxWidth(90);
			}
		}

		//Status bar
		statusLabel = new JLabel("");
		queryProgress = new JProgressBar(JProgressBar.HORIZONTAL, 0, 1);
		queryProgress.setStringPainted(true);
		JPanel statusBar = new JPanel(new GridLayout(1, 2));
		statusBar.add(statusLabel);
		statusBar.add(queryProgress);

		//Refresh Buttons
		JPanel refreshPanel = new JPanel();
		refresh[0] = new JButton("Full Refresh");
		refresh[0].setActionCommand("Full Refresh");
		refresh[1] = new JButton("Refresh All");
		refresh[1].setActionCommand("Refresh All");
		refresh[2] = new JButton("Refresh Current");
		refresh[2].setActionCommand("Refresh Current");
		for(int i = 0;i < refresh.length;i++)
		{
			refresh[i].addActionListener(this);
			refreshPanel.add(refresh[i]);
		}

		//Menu Bar
		menuBar = new JMenuBar();
		menu = new JMenu[] {
			new JMenu("Browser"),
			new JMenu("Help"),
			new JMenu("Edit")
		};
		menuItem = new JMenuItem[] {
			new JMenuItem("Exit"),
			new JMenuItem("About"),
			new JMenuItem("Configure"),
			new JMenuItem("Copy Server IP"),
			new JMenuItem("Copy Server Name")
		};
		for(JMenuItem item : menuItem) //All menu items have this as their action listener
			item.addActionListener(this);
		menuItem[0].setActionCommand("Exit");
		menuItem[1].setActionCommand("About");
		menuItem[2].setActionCommand("Configure");
		menuItem[3].setActionCommand("Copy Server IP");
		menuItem[3].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
		menuItem[4].setActionCommand("Copy Server Name");
		//Scope the menus for extra readability
		menuBar.add(menu[0]);
		{
			menu[0].add(menuItem[2]);
			menu[0].addSeparator();
			menu[0].add(menuItem[0]);
		}
		menuBar.add(menu[2]);
		{
			menu[2].add(menuItem[4]);
			menu[2].add(menuItem[3]);
		}
		menuBar.add(menu[1]);
		{
			menu[1].add(menuItem[1]);
		}

		//Set up frame
		frame.setJMenuBar(menuBar);
		frame.add(refreshPanel, BorderLayout.NORTH);
		frame.add(serverListPane);
		frame.add(statusBar, BorderLayout.SOUTH);
	}

	public void run()
	{
		frame.setVisible(true);
	}
	public void updateList(boolean newServer, int update)
	{
		frame.repaint();
		try
		{
			if(newServer)
				serverList.getRowSorter().rowsInserted(update, update);
			else
				serverList.getRowSorter().rowsUpdated(update, update);
		}
		catch(NullPointerException e) {} //This doesn't seem to cause any problems so lets ignore it.
		serverListPane.setViewportView(serverList);
	}
	public void setQueryProgress(int value, int max)
	{
		setQueryProgress(value, max, 0);
	}
	public void setQueryProgress(int value, int max, int serversFound)
	{
		String message = " players online.";
		if(Main.countPlayers() == 1)
			message = " player online.";
		statusLabel.setText(Main.countPlayers() + message);
		queryProgress.setMaximum(max);
		queryProgress.setValue(value);
		if(serversFound == 0)
			queryProgress.setString(value + "/" + max);
		else
			queryProgress.setString(serversFound + " Servers");
		queryProgress.repaint();
	}

	//Action Listener
	public void actionPerformed(ActionEvent e)
	{
		if(e.getActionCommand().equals("Full Refresh"))
			Main.cmd.setCommand(CommandThread.CMD_FULLREFRESH);
		else if(e.getActionCommand().equals("Refresh All"))
			Main.cmd.setCommand(CommandThread.CMD_REFRESHALL);
		else if(e.getActionCommand().equals("Refresh Current"))
		{
			if(serverList.getSelectedRow() != -1)
				Main.refreshServer(serverList.getRowSorter().convertRowIndexToModel(serverList.getSelectedRow()));
		}
		else if(e.getActionCommand().equals("Exit"))
			System.exit(0);
		else if(e.getActionCommand().equals("About"))
			new AboutDialog();
		else if(e.getActionCommand().equals("Configure"))
			new ConfigDialog();
		else if(e.getActionCommand().equals("Copy Server IP"))
		{
			if(serverList.getSelectedRow() != -1)
				Main.copyToClipboard(servers.get(serverList.getRowSorter().convertRowIndexToModel(serverList.getSelectedRow())).address());
		}
		else if(e.getActionCommand().equals("Copy Server Name"))
		{
			if(serverList.getSelectedRow() != -1)
				Main.copyToClipboard(servers.get(serverList.getRowSorter().convertRowIndexToModel(serverList.getSelectedRow())).name());
		}
	}

	//Window listener
	public void windowActivated(WindowEvent e){}
	public void windowClosed(WindowEvent e){}
	public void windowClosing(WindowEvent e)
	{
		Config.windowX = (int) frame.getLocation().getX();
		Config.windowY = (int) frame.getLocation().getY();
		Config.windowWidth = frame.getWidth();
		Config.windowHeight = frame.getHeight();
		Config.store();
	}
	public void windowDeactivated(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowIconified(WindowEvent e){}
	public void windowOpened(WindowEvent e){}
}